#ifndef SCIQLOP_VARIABLEACQUISITIONWORKER_H
#define SCIQLOP_VARIABLEACQUISITIONWORKER_H

#include "CoreGlobal.h"

#include <Data/DataProviderParameters.h>
#include <QLoggingCategory>
#include <QObject>
#include <QUuid>

#include <Data/AcquisitionDataPacket.h>
#include <Data/IDataSeries.h>
#include <Data/SqpRange.h>
#include <Data/VariableRequest.h>

#include <QLoggingCategory>

#include <Common/spimpl.h>

Q_DECLARE_LOGGING_CATEGORY(LOG_VariableAcquisitionWorker)

class Variable;
class IDataProvider;

/// This class aims to handle all acquisition request
class SCIQLOP_CORE_EXPORT VariableAcquisitionWorker : public QObject {
    Q_OBJECT
public:
    explicit VariableAcquisitionWorker(QObject *parent = 0);
    virtual ~VariableAcquisitionWorker();

    void pushVariableRequest(std::shared_ptr<Variable> variable, VariableRequest request);

    void abortProgressRequested(QUuid vIdentifier);

    void initialize();
    void finalize();
signals:
    void dataProvided(std::shared_ptr<Variable> variable, VariableRequest request);
    void variableRequestInProgress(QUuid vIdentifier, double progress);

public slots:
    void onDataAcquired(QUuid acquisitionId, std::shared_ptr<IDataSeries> dataSeries,
                        SqpRange range);
    void onVariableAcquisitionCanceled(QUuid acqIdentifier);
    void onVariableRetrieveDataInProgress(QUuid acqIdentifier, double progress);

private:
    void waitForFinish();

    class VariableAcquisitionWorkerPrivate;
    spimpl::unique_impl_ptr<VariableAcquisitionWorkerPrivate> impl;

private slots:
    void executeAcquisition(QUuid acquisitionId);
};

#endif // SCIQLOP_VARIABLEACQUISITIONWORKER_H
